@@ -226,7 +226,9 @@ def _check_chunks(x):
     return x
 
 
-def _get_dt_meta_computed(x, check_shape=True, check_graph=True, check_chunks=True):
+def _get_dt_meta_computed(
+    x, check_shape=True, check_graph=True, check_chunks=True, scheduler=None
+):
     x_original = x
     x_meta = None
     x_computed = None
@@ -240,7 +242,7 @@ def _get_dt_meta_computed(x, check_shape=True, check_graph=True, check_chunks=Tr
         if check_chunks:
             # Replace x with persisted version to avoid computing it twice.
             x = _check_chunks(x)
-        x = x.compute(scheduler="sync")
+        x = x.compute(scheduler=scheduler)
         x_computed = x
         if hasattr(x, "todense"):
             x = x.todense()
@@ -266,6 +268,7 @@ def assert_eq(
     check_meta=True,
     check_chunks=True,
     check_type=True,
+    scheduler="sync",
     **kwargs,
 ):
     a_original = a
@@ -277,10 +280,18 @@ def assert_eq(
         b = np.array(b)
 
     a, adt, a_meta, a_computed = _get_dt_meta_computed(
-        a, check_shape=check_shape, check_graph=check_graph, check_chunks=check_chunks
+        a,
+        check_shape=check_shape,
+        check_graph=check_graph,
+        check_chunks=check_chunks,
+        scheduler=scheduler,
     )
     b, bdt, b_meta, b_computed = _get_dt_meta_computed(
-        b, check_shape=check_shape, check_graph=check_graph, check_chunks=check_chunks
+        b,
+        check_shape=check_shape,
+        check_graph=check_graph,
+        check_chunks=check_chunks,
+        scheduler=scheduler,
     )
 
     if str(adt) != str(bdt):
