@@ -1,16 +1,18 @@
+import os
+import pathlib
+from time import sleep
+
 import numpy as np
 import pandas as pd
 import pytest
 
-import os
-from time import sleep
-import pathlib
-
 import dask
 import dask.dataframe as dd
 from dask.dataframe._compat import tm
-from dask.utils import tmpfile, tmpdir, dependency_depth
+from dask.dataframe.optimize import optimize_dataframe_getitem
 from dask.dataframe.utils import assert_eq
+from dask.layers import DataFrameIOLayer
+from dask.utils import dependency_depth, tmpdir, tmpfile
 
 
 def test_to_hdf():
@@ -120,8 +122,18 @@ def test_to_hdf_multiple_nodes():
     with tmpfile("h5") as fn:
         with pd.HDFStore(fn) as hdf:
             b.to_hdf(hdf, "/data*")
-            out = dd.read_hdf(fn, "/data*")
-            assert_eq(df16, out)
+        out = dd.read_hdf(fn, "/data*")
+        assert_eq(df16, out)
+
+    # Test getitem optimization
+    with tmpfile("h5") as fn:
+        a.to_hdf(fn, "/data*")
+        out = dd.read_hdf(fn, "/data*")[["x"]]
+        dsk = optimize_dataframe_getitem(out.dask, keys=out.__dask_keys__())
+        read = [key for key in dsk.layers if key.startswith("read-hdf")][0]
+        subgraph = dsk.layers[read]
+        assert isinstance(subgraph, DataFrameIOLayer)
+        assert subgraph.columns == ["x"]
 
 
 def test_to_hdf_multiple_files():
@@ -187,6 +199,16 @@ def test_to_hdf_multiple_files():
         out = dd.read_hdf(fn, "/data")
         assert_eq(df16, out)
 
+    # saving to multiple files where first file is longer
+    # https://github.com/dask/dask/issues/8023
+    with tmpdir() as dn:
+        fn1 = os.path.join(dn, "data_1.h5")
+        fn2 = os.path.join(dn, "data_2.h5")
+        b.to_hdf(fn1, "/data")
+        a.to_hdf(fn2, "/data")
+        out = dd.read_hdf([fn1, fn2], "/data")
+        assert_eq(pd.concat([df16, df]), out)
+
     # saving to multiple files with custom name_function
     with tmpdir() as dn:
         fn = os.path.join(dn, "data_*.h5")
@@ -203,8 +225,8 @@ def test_to_hdf_multiple_files():
     with tmpfile("h5") as fn:
         with pd.HDFStore(fn) as hdf:
             a.to_hdf(hdf, "/data*")
-            out = dd.read_hdf(fn, "/data*")
-            assert_eq(df, out)
+        out = dd.read_hdf(fn, "/data*")
+        assert_eq(df, out)
 
 
 def test_to_hdf_modes_multiple_nodes():
@@ -219,7 +241,7 @@ def test_to_hdf_modes_multiple_nodes():
         a.to_hdf(fn, "/data2")
         a.to_hdf(fn, "/data*", mode="a")
         out = dd.read_hdf(fn, "/data*")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
     # overwriting a file with a single partition
     a = dd.from_pandas(df, 1)
@@ -235,7 +257,7 @@ def test_to_hdf_modes_multiple_nodes():
         a.to_hdf(fn, "/data2")
         a.to_hdf(fn, "/data*", mode="a")
         out = dd.read_hdf(fn, "/data*")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
     # overwriting a file with two partitions
     a = dd.from_pandas(df, 2)
@@ -252,7 +274,7 @@ def test_to_hdf_modes_multiple_nodes():
         a.to_hdf(fn, "/data2")
         a.to_hdf(fn, "/data*", mode="a", append=False)
         out = dd.read_hdf(fn, "/data*")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
 
 def test_to_hdf_modes_multiple_files():
@@ -268,7 +290,7 @@ def test_to_hdf_modes_multiple_files():
         a.to_hdf(os.path.join(dn, "data2"), "/data")
         a.to_hdf(fn, "/data", mode="a")
         out = dd.read_hdf(fn, "/data*")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
     # appending two partitions to existing data
     a = dd.from_pandas(df, 2)
@@ -277,7 +299,7 @@ def test_to_hdf_modes_multiple_files():
         a.to_hdf(os.path.join(dn, "data2"), "/data")
         a.to_hdf(fn, "/data", mode="a")
         out = dd.read_hdf(fn, "/data")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
     # overwriting a file with two partitions
     a = dd.from_pandas(df, 2)
@@ -295,7 +317,7 @@ def test_to_hdf_modes_multiple_files():
         a.to_hdf(os.path.join(dn, "data1"), "/data")
         a.to_hdf(fn, "/data", mode="a", append=False)
         out = dd.read_hdf(fn, "/data")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
 
 def test_to_hdf_link_optimizations():
