@@ -9,16 +9,16 @@ import sys
 import tempfile
 import uuid
 import warnings
-from _thread import RLock
-from collections.abc import Iterable, Iterator, Mapping
+from collections.abc import Hashable, Iterable, Iterator, Mapping
 from contextlib import contextmanager, nullcontext, suppress
 from datetime import datetime, timedelta
 from errno import ENOENT
 from functools import lru_cache
 from importlib import import_module
 from numbers import Integral, Number
-from threading import Lock
-from typing import TypeVar
+from operator import add
+from threading import Lock, RLock
+from typing import ClassVar, TypeVar
 from weakref import WeakValueDictionary
 
 import tlz as toolz
@@ -153,15 +153,6 @@ def ndeepmap(n, func, seq):
         return func(seq)
 
 
-@_deprecated(
-    version="2021.06.1", use_instead="contextlib.suppress from the standard library"
-)
-@contextmanager
-def ignoring(*exceptions):
-    with suppress(*exceptions):
-        yield
-
-
 def import_required(mod_name, error_msg):
     """Attempt to import a required dependency.
 
@@ -282,15 +273,6 @@ def tmp_cwd(dir=None):
             yield dirname
 
 
-@_deprecated(
-    version="2021.06.1", use_instead="contextlib.nullcontext from the standard library"
-)
-@contextmanager
-def noop_context():
-    with nullcontext():
-        yield
-
-
 class IndexCallable:
     """Provide getitem syntax for functions
 
@@ -750,6 +732,14 @@ def _derived_from(cls, method, ua_args=None, extra="", skipblocks=0):
     if doc is None:
         doc = ""
 
+    # pandas DataFrame/Series sometimes override methods without setting __doc__
+    if not doc and cls.__name__ in {"DataFrame", "Series"}:
+        for obj in cls.mro():
+            obj_method = getattr(obj, method.__name__, None)
+            if obj_method is not None and obj_method.__doc__:
+                doc = obj_method.__doc__
+                break
+
     # Insert disclaimer that this is a copied docstring
     if doc:
         doc = ignore_warning(
@@ -934,7 +924,7 @@ def digit(n, k, base):
     >>> digit(1234, 3, 10)
     1
     """
-    return n // base ** k % base
+    return n // base**k % base
 
 
 def insert(tup, loc, val):
@@ -996,7 +986,7 @@ def put_lines(buf, lines):
     buf.write("\n".join(lines))
 
 
-_method_cache = {}
+_method_cache: dict[str, methodcaller] = {}
 
 
 class methodcaller:
@@ -1008,7 +998,12 @@ class methodcaller:
     """
 
     __slots__ = ("method",)
-    func = property(lambda self: self.method)  # For `funcname` to work
+    method: str
+
+    @property
+    def func(self) -> str:
+        # For `funcname` to work
+        return self.method
 
     def __new__(cls, method: str):
         try:
@@ -1060,16 +1055,18 @@ class MethodCache:
     True
     """
 
-    __getattr__ = staticmethod(methodcaller)
-    __dir__ = lambda self: list(_method_cache)
+    def __getattr__(self, item):
+        return methodcaller(item)
+
+    def __dir__(self):
+        return list(_method_cache)
 
 
 M = MethodCache()
 
 
 class SerializableLock:
-    _locks = WeakValueDictionary()
-    """ A Serializable per-process Lock
+    """A Serializable per-process Lock
 
     This wraps a normal ``threading.Lock`` object and satisfies the same
     interface.  However, this lock can also be serialized and sent to different
@@ -1096,7 +1093,11 @@ class SerializableLock:
     The creation of locks is itself not threadsafe.
     """
 
-    def __init__(self, token=None):
+    _locks: ClassVar[WeakValueDictionary[Hashable, Lock]] = WeakValueDictionary()
+    token: Hashable
+    lock: Lock
+
+    def __init__(self, token: Hashable | None = None):
         self.token = token or str(uuid.uuid4())
         if self.token in SerializableLock._locks:
             self.lock = SerializableLock._locks[self.token]
@@ -1403,16 +1404,16 @@ def parse_bytes(s):
 
 
 byte_sizes = {
-    "kB": 10 ** 3,
-    "MB": 10 ** 6,
-    "GB": 10 ** 9,
-    "TB": 10 ** 12,
-    "PB": 10 ** 15,
-    "KiB": 2 ** 10,
-    "MiB": 2 ** 20,
-    "GiB": 2 ** 30,
-    "TiB": 2 ** 40,
-    "PiB": 2 ** 50,
+    "kB": 10**3,
+    "MB": 10**6,
+    "GB": 10**9,
+    "TB": 10**12,
+    "PB": 10**15,
+    "KiB": 2**10,
+    "MiB": 2**20,
+    "GiB": 2**30,
+    "TiB": 2**40,
+    "PiB": 2**50,
     "B": 1,
     "": 1,
 }
@@ -1531,11 +1532,11 @@ def format_bytes(n: int) -> str:
     For all values < 2**60, the output is always <= 10 characters.
     """
     for prefix, k in (
-        ("Pi", 2 ** 50),
-        ("Ti", 2 ** 40),
-        ("Gi", 2 ** 30),
-        ("Mi", 2 ** 20),
-        ("ki", 2 ** 10),
+        ("Pi", 2**50),
+        ("Ti", 2**40),
+        ("Gi", 2**30),
+        ("Mi", 2**20),
+        ("ki", 2**10),
     ):
         if n >= k * 0.9:
             return f"{n / k:.2f} {prefix}B"
@@ -1809,11 +1810,11 @@ def stringify_collection_keys(obj):
 try:
     _cached_property = functools.cached_property
 except AttributeError:
-    # TODO: Copied from functools.cached_property in python 3.8. Remove when minimum
-    # supported python version is 3.8:
+    # TODO: Copied from functools.cached_property in python 3.8.
+    #       Remove when minimum supported python version is 3.8:
     _NOT_FOUND = object()
 
-    class _cached_property:
+    class _cached_property:  # type: ignore
         def __init__(self, func):
             self.func = func
             self.attrname = None
@@ -1868,3 +1869,62 @@ class cached_property(_cached_property):
     def __set__(self, instance, val):
         """Raise an error when attempting to set a cached property."""
         raise AttributeError("Can't set attribute")
+
+
+class _HashIdWrapper:
+    """Hash and compare a wrapped object by identity instead of value"""
+
+    def __init__(self, wrapped):
+        self.wrapped = wrapped
+
+    def __eq__(self, other):
+        if not isinstance(other, _HashIdWrapper):
+            return NotImplemented
+        return self.wrapped is other.wrapped
+
+    def __ne__(self, other):
+        if not isinstance(other, _HashIdWrapper):
+            return NotImplemented
+        return self.wrapped is not other.wrapped
+
+    def __hash__(self):
+        return id(self.wrapped)
+
+
+@functools.lru_cache()
+def _cumsum(seq, initial_zero):
+    if isinstance(seq, _HashIdWrapper):
+        seq = seq.wrapped
+    if initial_zero:
+        return tuple(toolz.accumulate(add, seq, 0))
+    else:
+        return tuple(toolz.accumulate(add, seq))
+
+
+def cached_cumsum(seq, initial_zero=False):
+    """Compute :meth:`toolz.accumulate` with caching.
+
+    Caching is by the identify of `seq` rather than the value. It is thus
+    important that `seq` is a tuple of immutable objects, and this function
+    is intended for use where `seq` is a value that will persist (generally
+    block sizes).
+
+    Parameters
+    ----------
+    seq : tuple
+        Values to cumulatively sum.
+    initial_zero : bool, optional
+        If true, the return value is prefixed with a zero.
+
+    Returns
+    -------
+    tuple
+    """
+    if isinstance(seq, tuple):
+        # Look up by identity first, to avoid a linear-time __hash__
+        # if we've seen this tuple object before.
+        result = _cumsum(_HashIdWrapper(seq), initial_zero)
+    else:
+        # Construct a temporary tuple, and look up by value.
+        result = _cumsum(tuple(seq), initial_zero)
+    return result
