@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import importlib
+import time
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
@@ -19,6 +20,11 @@ def add(x, y):
     return x + y
 
 
+def slowadd(a, b, delay=0.1):
+    time.sleep(delay)
+    return a + b
+
+
 class GetFunctionTestMixin:
     """
     The GetFunctionTestCase class can be imported and used to test foreign
